var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "request", "xml-js", "../vast-element", "browser-or-node", "./logs"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var request = require("request");
    var xml_js_1 = require("xml-js");
    var vast_element_1 = require("../vast-element");
    var browser_or_node_1 = require("browser-or-node");
    var logs_1 = require("./logs");
    function buildVast(current, currentTag) {
        /* istanbul ignore next */
        if (current && current.elements) {
            if (current.elements.length === 1 &&
                (current.elements[0].text || current.elements[0].cdata)) {
                var currentTmp = current.elements[0];
                var currentText = String(currentTmp.text || currentTmp.cdata);
                currentTag.content = currentText;
            }
            else {
                var currentChild = void 0;
                for (var i = 0; i < current.elements.length; i++) {
                    var currentTmp = current.elements[i];
                    // TODO refacto attachCustomTag from real tag for better integration
                    // and fallback on dangerous
                    currentChild = currentTag.attachCustomTag(currentTmp.name, String(currentTmp.attributes));
                    buildVast(currentTmp, currentChild);
                }
            }
        }
    }
    exports.buildVast = buildVast;
    function fetchUrl(_a) {
        var url = _a.url, _b = _a.loadCallback, loadCallback = _b === void 0 ? function () { } : _b, _c = _a.syncInBrowser, syncInBrowser = _c === void 0 ? false : _c;
        if (!url) {
            throw new Error("'url' is undefined");
        }
        var fail = function () {
            throw new Error(url + " fetch failed");
        };
        if (browser_or_node_1.isBrowser) {
            var fetchReq = new XMLHttpRequest();
            if (syncInBrowser) {
                fetchReq.open("GET", url, false);
                fetchReq.send();
                if (fetchReq.status === 200) {
                    loadCallback(fetchReq.responseText);
                    return fetchReq.responseText;
                }
                else {
                    fail();
                }
            }
            else {
                fetchReq.open("GET", url, true);
                fetchReq.onerror = fail;
                fetchReq.onload = function (res) {
                    // tslint:disable-next-line:no-console
                    console.log(res);
                    // loadCallback(res.responseText);
                };
                fetchReq.send();
            }
            throw new Error(url + " was not found");
        }
        else if (browser_or_node_1.isNode) {
            // tslint:disable:no-console
            request(url, function (error, response, body) {
                if (error) {
                    fail();
                }
                loadCallback(body);
            });
        }
        else {
            throw new Error("Not supported environment");
        }
    }
    // TODO this is only exported for test, it should not be
    function createVastWithBuilder(vastRawCode, options) {
        if (options === void 0) { options = {}; }
        options = __assign({ logWarn: false }, options);
        var parsedXml;
        try {
            parsedXml = xml_js_1.xml2js(vastRawCode);
        }
        catch (e) {
            logs_1.warnOrThrow("Error during the vast parsing, it seems not valid XML", options);
        }
        return createVastFromJson(parsedXml);
    }
    exports.createVastWithBuilder = createVastWithBuilder;
    function createVastFromJson(vastJsonCode, options) {
        if (options === void 0) { options = {}; }
        options = __assign({ logWarn: false }, options);
        var root = new vast_element_1.default();
        root.parseOptions(options);
        buildVast(vastJsonCode, root);
        return root;
    }
    function downloadVastAndWrappersSync(vastUrl, options) {
        var vastAndWrappers = [];
        var currentVast;
        do {
            var vastRawContent = fetchUrl({ url: vastUrl, syncInBrowser: true });
            currentVast = createVastWithBuilder(vastRawContent);
            vastAndWrappers.push(currentVast);
            if (currentVast.isWrapper()) {
                var VASTAdTagURI = currentVast.get(["VASTAdTagURI"])[0];
                if (VASTAdTagURI) {
                    vastUrl = VASTAdTagURI.getContent();
                }
            }
        } while (currentVast.isWrapper());
        return vastAndWrappers;
    }
    exports.downloadVastAndWrappersSync = downloadVastAndWrappersSync;
    function downloadVastAndWrappersAsync(vastUrl, options, callback, actualDownloadedVasts) {
        var vastAndWrappers = actualDownloadedVasts || [];
        var currentVast;
        fetchUrl({
            loadCallback: function (vastRawContent) {
                currentVast = createVastWithBuilder(vastRawContent);
                vastAndWrappers.push(currentVast);
                if (currentVast.isWrapper()) {
                    var VASTAdTagURI = currentVast.get(["VASTAdTagURI"])[0];
                    if (VASTAdTagURI) {
                        vastUrl = VASTAdTagURI.getContent();
                    }
                    downloadVastAndWrappersAsync(vastUrl, options, callback, vastAndWrappers);
                }
                else {
                    callback(vastAndWrappers);
                }
            },
            syncInBrowser: true,
            url: vastUrl
        });
    }
    exports.downloadVastAndWrappersAsync = downloadVastAndWrappersAsync;
});
//# sourceMappingURL=vast.js.map