(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "array-flatten", "browser-or-node", "../common/utils/checks", "../common/utils/logs", "../common/utils/vast"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var flatten = require("array-flatten");
    var browser_or_node_1 = require("browser-or-node");
    var checks_1 = require("../common/utils/checks");
    var logs_1 = require("../common/utils/logs");
    var vast_1 = require("../common/utils/vast");
    var VastParser = /** @class */ (function () {
        function VastParser(vastUrl, options) {
            if (options === void 0) { options = {}; }
            this.originalVastUrl = vastUrl;
            this.options = options;
        }
        VastParser.prototype.parseAsync = function (callback) {
            var _this = this;
            if (this.parsed) {
                callback(this);
            }
            vast_1.downloadVastAndWrappersAsync(this.originalVastUrl, this.options, function (vasts) {
                _this.vasts = vasts;
                _this.parsed = true;
                callback(_this);
            });
        };
        VastParser.prototype.parseSync = function () {
            if (browser_or_node_1.isNode) {
                throw new Error("parseSync is only available in a browser context");
            }
            if (this.parsed) {
                return this;
            }
            this.vasts = vast_1.downloadVastAndWrappersSync(this.originalVastUrl, this.options);
            this.parsed = true;
            return this;
        };
        VastParser.prototype.getVasts = function () {
            return this.vasts || [];
        };
        VastParser.prototype.getVastElements = function (arrayOfTagNames) {
            return this.get(arrayOfTagNames);
        };
        VastParser.prototype.getContents = function (arrayOfTagNames) {
            return this.get(arrayOfTagNames, "content");
        };
        VastParser.prototype.getAttributes = function (arrayOfTagNames, attribute) {
            return this.get(arrayOfTagNames, attribute);
        };
        VastParser.prototype.get = function (arrayOfTagNames, details) {
            var _this = this;
            var vastElements = flatten(this.vasts.map(function (v) { return v.get(arrayOfTagNames, true); }));
            if (details) {
                return vastElements.map(function (vastElement) {
                    if (details === "content") {
                        if (checks_1.isNull(vastElement.content)) {
                            logs_1.warnOrThrow(vastElement.name + " does not have content", _this.options);
                        }
                        return vastElement.content;
                    }
                    else {
                        var attributeLowerCased_1 = {};
                        Object.keys(vastElement.attrs).forEach(function (attr) {
                            attributeLowerCased_1[attr.toLowerCase()] = vastElement.attrs[attr];
                        });
                        if (checks_1.isNull(attributeLowerCased_1[details.toLowerCase()])) {
                            logs_1.warnOrThrow(vastElement.name + " does not have content", _this.options);
                        }
                        return attributeLowerCased_1[details.toLowerCase()];
                    }
                });
            }
            return vastElements;
        };
        return VastParser;
    }());
    exports.default = VastParser;
});
//# sourceMappingURL=index.js.map