var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "xml-js", "../common/utils/logs", "../common/utils/validate-node", "../common/utils/vast", "../common/vast-element", "../generated/api/vast2_0-validator", "../generated/api/vast3_0-validator", "../generated/api/vast4_0-validator", "../generated/api/vast4_1-validator"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var xml_js_1 = require("xml-js");
    var logs_1 = require("../common/utils/logs");
    var validate_node_1 = require("../common/utils/validate-node");
    var vast_1 = require("../common/utils/vast");
    var vast_element_1 = require("../common/vast-element");
    var vast2_0_validator_1 = require("../generated/api/vast2_0-validator");
    var vast3_0_validator_1 = require("../generated/api/vast3_0-validator");
    var vast4_0_validator_1 = require("../generated/api/vast4_0-validator");
    var vast4_1_validator_1 = require("../generated/api/vast4_1-validator");
    var validators = {
        "2_0": vast2_0_validator_1.vastValidator2_0,
        "3_0": vast3_0_validator_1.vastValidator3_0,
        "4_0": vast4_0_validator_1.vastValidator4_0,
        "4_1": vast4_1_validator_1.vastValidator4_1
    };
    function parseVast(vastRawCode, options) {
        if (options === void 0) { options = {}; }
        options = __assign({ logWarn: false }, options);
        var parsedXml;
        try {
            parsedXml = xml_js_1.xml2js(vastRawCode);
        }
        catch (error) {
            if (options.logWarn) {
                logs_1.logError("Error during the vast parsing, it seems not valid XML");
            }
            return null;
        }
        var root = new vast_element_1.default();
        root.parseOptions(options);
        // this modify root
        vast_1.buildVast(parsedXml, root);
        return root;
    }
    var VastValidator = /** @class */ (function () {
        function VastValidator(vast, options) {
            if (options === void 0) { options = {}; }
            if (typeof vast === "string") {
                vast = parseVast(vast, options);
            }
            this.vastRoot = vast.getRoot();
        }
        // > validate your current vast build. print error if options.logWarn = true
        // * validate(): boolean
        VastValidator.prototype.validate = function () {
            var validator = validators[this.vastRoot.getVastSnakeVersion()].validator;
            return validate_node_1.validateNext(this.vastRoot, validator);
        };
        return VastValidator;
    }());
    exports.default = VastValidator;
});
//# sourceMappingURL=index.js.map